package org.gyan;

/****
 GSSClient.java
 ****/

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.Properties;

import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;

import org.ietf.jgss.GSSContext;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSManager;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.MessageProp;
import org.ietf.jgss.Oid;

public class GSSClient implements java.security.PrivilegedAction<byte[]> {

	// Handles callback from the JAAS framework.
	private BeanCallbackHandler beanCallbackHandler = null;

	// The main object that handles all JAAS login.
	LoginContext peerLC = null;

	// Socket and streams used for communication.
	Socket socket = null;
	DataInputStream inStream;
	DataOutputStream outStream;

	// This and remote clients.
	String clientName = null;
	String serverName = null;

	// Address and port of remote server.
	String serverAddress = null;
	int serverPort;

	// The name of the client configuration.
	String confName = null;

	private GSSContext clientGSSContext = null;

	/**
	 * @param clientName, Client SPN 
	 * @param password, client password
	 */
	public GSSClient(String clientName, String password){
		Properties env = new Properties();
		try {
			env.load(this.getClass().getClassLoader().getResourceAsStream("gssclient.properties"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.clientName = clientName;
		this.serverName = (String)env.getProperty("Server.SPN");
		this.serverAddress = (String)env.getProperty("Server.Address");
		this.confName = (String)env.getProperty("JAAS.Config.Name");
		
		beanCallbackHandler = new BeanCallbackHandler(clientName, password);
		setSystemProperties((String)env.getProperty("KDC.Realm"),
				(String)env.getProperty("KDC.Address"),
				(String)env.getProperty("JAAS.Config.Path"));
	}
	/**
	 * The GSSClient constructor only sets all the required parameters.
	 * 
	 * @param clientName, Client SPN 
	 * @param password, client password
	 * @param serverName, Remote server/service SPN
	 * @param serverAddress, Remote Server address IP or Hostname 
	 * @param kerberosRealm, KDC realm or domain name
	 * @param kdcAddress, KDC Server address IP or Hostname 
	 * @param confFile, fll path of JAAS COnfig file
	 * @param confName, JAAS config name
	 */
	public GSSClient(String clientName, String password, String serverName,
			String serverAddress, String kerberosRealm,
			String kdcAddress, String confFile, String confName){
		// The beanCallbackHandler will require the name and password of the
		// client.
		beanCallbackHandler = new BeanCallbackHandler(clientName, password);
		this.clientName = clientName;
		this.serverName = serverName;
		this.serverAddress = serverAddress;
		this.confName = confName;
		setSystemProperties(kerberosRealm, kdcAddress, confFile);
	}// KerberoseLoginBean
	
	private void setSystemProperties(String kerberosRealm, String kdcAddress,
			String confFile) {
		System.setProperty("java.security.krb5.realm", kerberosRealm);
		System.setProperty("java.security.krb5.kdc", kdcAddress);
		System.setProperty("java.security.auth.login.config", confFile);
		System.setProperty("sun.security.krb5.debug", "true");
	}

	public byte[] login() {
		try {
			peerLC = new LoginContext(confName, beanCallbackHandler);
			peerLC.login();
			return Subject.doAs(peerLC.getSubject(), this);
		} catch (Exception e) {
			System.out
					.println(">>>> GSSClient....Secure Context not established..");
			e.printStackTrace();
			return null;
		}// catch

	}// establishSecureContextWithServer

	// This is the only method in PrivilegedAction interface.
	// It receives control only in case of successful authentication of the
	// client.
	public byte[] run() {

		try {
			GSSManager manager = GSSManager.getInstance();
			Oid kerberos = new Oid("1.2.840.113554.1.2.2");
			GSSName clientPeerName = manager.createName(
			// Name of the client for which we want to create this GSSName
			// object.
					clientName,
					// Type of GSSName. Our client is a Windows user,
					// which we can specifiy using GSSName.NT_USER_NAME
					// property.
					GSSName.NT_USER_NAME);
			// GSSName remotePeerName2 = manager.createName("FakeSERver",
			// GSSName.NT_USER_NAME);
			GSSName remotePeerName = manager.createName(serverName,
					GSSName.NT_USER_NAME);
			System.out.println(">>> GSSClient... Getting client credentials");

			GSSCredential peerCredentials = manager.createCredential(
			// The GSSName object of the client.
					clientPeerName,
					// Time for which credentials whill be valid.
					10 * 600,
					// Kerberos mecahnism identifier.
					kerberos,
					// The client only intiates the secure context request.
					GSSCredential.INITIATE_ONLY);

			System.out
					.println(">>> GSSClient...GSSManager creating security context");
			clientGSSContext= manager.createContext(remotePeerName, kerberos,
					peerCredentials, GSSContext.DEFAULT_LIFETIME);

			// peerContext.requestCredDeleg(true);
			clientGSSContext.requestConf(true);
			clientGSSContext.requestMutualAuth(false);// I don't want to authenticate the 
														// Server.
			byte[] byteToken = new byte[0];

			byteToken = clientGSSContext.initSecContext(byteToken, 0,
						byteToken.length);
			return byteToken;
		}// try
		catch (org.ietf.jgss.GSSException ge) {
			ge.printStackTrace();
			System.out.println(">>> GSSClient... GSS Exception "
					+ ge.getMessage());
		}

		catch (java.lang.Exception e) {
			e.printStackTrace();
			System.out.println(">>> GSSClient...  Exception " + e.getMessage());
		}// catch
		return null;
	}// run

	// It returns the established login context to client.
	public LoginContext getLoginContext() {
		return peerLC;
	}// getloginContext

	public byte[] wrapMessage(byte[] msg) throws GSSException {
		MessageProp msgProp = new MessageProp(0, false);
		return clientGSSContext.wrap(msg, 0, msg.length, msgProp);
	}

	public byte[] unWrapMessage(byte[] msg) throws GSSException {
		MessageProp msgProp = new MessageProp(0, false);
		return clientGSSContext.unwrap(msg, 0, msg.length, msgProp);
	}

}// GSSClient

