package org.gyan.server;

/****
 GSSServer.java
 ****/

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.util.Properties;

import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;

import org.gyan.BeanCallbackHandler;
import org.ietf.jgss.GSSContext;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSManager;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.MessageProp;
import org.ietf.jgss.Oid;

public class GSSServer implements java.security.PrivilegedAction {

	// Handles callback from the JAAS framework.
	BeanCallbackHandler beanCallbackHandler = null;

	// The main object that handles all JAAS login.
	LoginContext serverLC = null;

	// The context for secure communication with client.
	GSSContext serverGSSContext = null;

	// Socket and streams used for communication.
	ServerSocket serverSocket = null;
	DataInputStream inStream = null;
	DataOutputStream outStream = null;

	// Name and port of server.
	String serverName = null;
	int serverPort;

	// Configuration file and the name of the client configuration.
	String confFile = null;
	String confName = null;

	private String clientName;

	public static void main(String[] args) throws IOException, GSSException {
		// Starting the server.
		new GSSServer().startServer();

	}// main

	// GSSServer constructor
	public GSSServer() {
		Properties env = new Properties();
		try {
			env.load(this.getClass().getClassLoader()
					.getResourceAsStream("gssserver.properties"));
		} catch (IOException e) {
			e.printStackTrace();
		}

		this.serverName = (String) env.getProperty("Server.SPN");
		this.confName = (String) env.getProperty("JAAS.Config.Name");

		beanCallbackHandler = new BeanCallbackHandler(this.serverName,
				(String) env.getProperty("Server.Password"));
		setSystemProperties((String) env.getProperty("KDC.Realm"),
				(String) env.getProperty("KDC.Address"),
				(String) env.getProperty("JAAS.Config.Path"));
	}// GSSServer

	private void setSystemProperties(String kerberosRealm, String kdcAddress,
			String confFile) {
		System.setProperty("java.security.krb5.realm", kerberosRealm);
		System.setProperty("java.security.krb5.kdc", kdcAddress);
		System.setProperty("java.security.auth.login.config", confFile);
		System.setProperty("sun.security.krb5.debug", "true");
	}

	public boolean startServer() {
		try {
			serverLC = new LoginContext(confName, beanCallbackHandler);
			serverLC.login();
			Subject.doAs(serverLC.getSubject(), this);
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			System.out
					.println(">>> GSSServer...Secure Context not established..");
			return false;
		}// catch

	}// start

	public Object run() {
		try {
			GSSManager manager = GSSManager.getInstance();
			Oid kerberos = new Oid("1.2.840.113554.1.2.2");
			GSSName serverGSSName = manager.createName(serverName,
					GSSName.NT_USER_NAME);
			GSSCredential serverGSSCreds = manager.createCredential(
					serverGSSName, GSSCredential.INDEFINITE_LIFETIME, kerberos,
					// The server accepts secure context request.
					GSSCredential.ACCEPT_ONLY);
			serverGSSContext = manager.createContext(serverGSSCreds);
		}// try
		catch (java.lang.Exception e) {
			e.printStackTrace();
		}
		return null;
	}// run

	public void authenticate(byte[] kerberosToken) throws GSSException {
		if (!serverGSSContext.isEstablished()) {
			serverGSSContext.acceptSecContext(kerberosToken, 0, kerberosToken.length);
			clientName = serverGSSContext.getTargName().toString();
		}
	}

	public byte[] wrapMessage(byte[] msg) throws GSSException {
		MessageProp msgProp = new MessageProp(0, false);
		return serverGSSContext.wrap(msg, 0, msg.length, msgProp);
	}

	public byte[] unWrapMessage(byte[] msg) throws GSSException {
		MessageProp msgProp = new MessageProp(0, false);
		return serverGSSContext.unwrap(msg, 0, msg.length, msgProp);
	}

	public void closeServerContext() throws GSSException {
		// Disposing and closing client and server sockets.
		serverGSSContext.dispose();
	}

	public String getClientName() {
		return clientName;
	}
}// GSSServer
